#!/usr/bin/env python
# -*- coding: utf8 -*-

#-----------------------------------------
# This is a part of Collège 3D project.
# Name:         messages.py
# Author:       Pascal Peter
# Copyright:    (C) 2003-2013 Pascal Peter
# Licence:      GNU General Public Licence version 3
# Website:      http://pascal.peter.free.fr/
# Email:        pascal.peter at free.fr
#-----------------------------------------

"""
DESCRIPTION :
    une application indépendante pour saisir un message.
    Permet de contourner la limitation du clavier dans le BGE.
    Le message est enregistré dans le dossier de config de l'application.
"""

# importation des modules utiles :
from __future__ import print_function
import sys
import os
import csv



# **************************************************
#       POUR FONCTIONNER AVEC PYTHON 2.x OU 3.x
# **************************************************
PYTHONVERSION = sys.version_info[0] * 10 + sys.version_info[1]



# **************************************************
#       GESTION DES DOSSIERS ET CHEMINS
# **************************************************
# récupération du chemin de lancement :
HERE = os.path.dirname(sys.argv[0])
sys.path.insert(0, HERE)
os.chdir(HERE)

def findConfigDir(appShortName='college3d'):
    """
    fonction de recherche du dossier de configuration
    """
    configDir = ''
    if sys.platform == 'win32':
        configDir = os.environ['APPDATA']
        configDir = os.path.join(configDir, appShortName)
    else:
        configsDir = os.path.join(os.path.expanduser('~'), '.config')
        if os.path.isdir(configsDir):
            configDir = os.path.join(configsDir, appShortName)
        else:
            configDir = os.path.join(os.path.expanduser('~'), '.{0}'.format(appShortName))
    # on vérifie l'existence du dossier :
    if not(os.path.isdir(configDir)):
        os.mkdir(configDir)
    if not(os.path.isdir(configDir)):
        configDir = '{1}{0}..'.format(os.sep, HERE)
        configDir = os.path.abspath(configDir)
    return configDir

# fichier de sauvegarde du message :
messageFileName = os.path.join(findConfigDir(), 'message.txt')

def saveText(text=''):
    """
    on enregistre le texte dans le fichier messageFileName
    """
    if PYTHONVERSION >= 30:
        messageFile = open(messageFileName, 'w', newline='', encoding='utf-8')
        messageFile.writelines(text)
    else:
        messageFile = open(messageFileName, "wb")
        try:
            text = text.encode('utf8')
        except:
            if not(isinstance(text, str)):
                try:
                    text = str(text)
                except:
                    print('ERROR:', type(text))
        messageFile.writelines(text)
    messageFile.close()



# **************************************************
#       I18N
# **************************************************
# récupération de la langue (passée en paramètre, par exemple LANG=fr)
LANG = ''
for arg in sys.argv:
    if arg.split('=')[0] == 'LANG':
        LANG = arg.split('=')[1]

def u(text):
    """
    retourne une version unicode de text
    """
    if PYTHONVERSION >= 30:
        try:
            if isinstance(text, str):
                return text
            else:
                return str(text)
        except:
            print('ERROR unicode | {0} | {1}'.format(type(text), text))
            return text
    else:
        try:
            return unicode(text)
        except:
            if isinstance(text, str):
                return text.decode('utf-8')
            else:
                print('ERROR unicode | {0} | {1}'.format(type(text), text))
                return text

TRANSLATIONS = {}
def openCsvFile(csvFile='divers'):
    """
    récupération des traductions depuis le fichier divers.csv
    """
    global TRANSLATIONS
    languages = []
    csvFileName = '{1}{0}..{0}translations{0}{2}.csv'.format(os.sep, HERE, csvFile)
    csvFileName = os.path.abspath(csvFileName)
    try:
        nbCols = 0
        if PYTHONVERSION >= 30:
            theFile = open(csvFileName, newline='', encoding='utf-8')
        else:
            theFile = open(csvFileName, "rb")
        reader = csv.reader(theFile, 
                            delimiter=';', 
                            quotechar='"', 
                            quoting=csv.QUOTE_NONNUMERIC)
        firstRow = True
        for row in reader:
            if firstRow:
                firstRow = False
                for lang in row[1:]:
                    languages.append(lang)
                TRANSLATIONS['languages'] = languages
                nbCols = len(languages)
            else:
                text = row[0]
                TRANSLATIONS[text] = {}
                for numCol in range(nbCols):
                    lang = languages[numCol]
                    translation = row[numCol + 1]
                    TRANSLATIONS[text][lang] = u(translation)
    except:
        print('Failed to open {0}'.format(csvFileName))
    finally:
        theFile.close()

openCsvFile()

def tr(text):
    """
    traduit le texte dans la langue définie.
    Si la traduction n'est pas trouvée, on retourne le texte de départ.
    """
    result = text
    # on lit la traduction :
    if text in TRANSLATIONS:
        if LANG in TRANSLATIONS[text]:
            result = TRANSLATIONS[text][LANG]
    if result == '':
        result = text
    return result

# traduction des textes utiles à l'interface :
windowTitleText = tr('Write a message')
cancelButtonText = tr('Cancel')
okButtonText = tr('OK')



# **************************************************
#       DÉTECTION DE LA GUI DISPONIBLE
#       DANS L'ORDRE : PYSIDE, PYQT PUIS TKINTER
# **************************************************
GUI, ARG_GUI = '', ''
# pour tests (variable GUI passée en argument, par exemple GUI=TKINTER) :
for arg in sys.argv:
    if arg.split('=')[0] == 'GUI':
        ARG_GUI = arg.split('=')[1]
try:
    from PySide import QtCore, QtGui
    GUI = 'PYSIDE'
except:
    pass
if (GUI == '') or (ARG_GUI == 'PYQT'):
    try:
        import sip
        sip.setapi('QString', 2)
        sip.setapi('QVariant', 2)
        from PyQt4 import QtCore, QtGui
        GUI = 'PYQT'
    except:
        pass
if (GUI == '') or (ARG_GUI == 'TKINTER'):
    try:
        if PYTHONVERSION >= 30:
            import tkinter
        else:
            import Tkinter as tkinter
        GUI = 'TKINTER'
    except:
        pass
#print('GUI : {0}'.format(GUI))



# **************************************************
#       DIVERS
# **************************************************
windowWidth = 400
windowHeight = 100



# **************************************************
#       CRÉATION DE LA FENÊTRE (SELON LA GUI)
# **************************************************
if GUI in ('PYSIDE', 'PYQT'):
    class MyWindow(QtGui.QDialog):
        def __init__(self):
            super(MyWindow, self).__init__()

            # un QTextEdit pour saisir le texte :
            self.editor = QtGui.QTextEdit(self)

            # les boutons OK et Annuler :
            buttonBox = QtGui.QDialogButtonBox(self)
            okButton = QtGui.QPushButton(okButtonText)
            okButton.clicked.connect(self.doOk)
            buttonBox.addButton(okButton, QtGui.QDialogButtonBox.ActionRole)
            cancelButton = QtGui.QPushButton(cancelButtonText)
            cancelButton.clicked.connect(self.close)
            buttonBox.addButton(cancelButton, QtGui.QDialogButtonBox.ActionRole)

            # on agence tout ça :
            layout = QtGui.QVBoxLayout(self)
            layout.addWidget(self.editor)
            layout.addWidget(buttonBox)

            # le titre de la fenêtre :
            self.setWindowTitle(windowTitleText)

            # redimensionnement de la fenêtre :
            self.setMinimumWidth(windowWidth)
            self.setMinimumHeight(windowHeight)

            # on vide le texte précédent :
            saveText()

        def doOk(self):
            saveText(self.editor.toPlainText())
            self.close()

elif GUI == 'TKINTER':
    class Application(tkinter.Frame):
        def __init__(self, master=None):
            tkinter.Frame.__init__(self, master)
            self.pack()

            # un Text pour saisir le texte :
            self.editor = tkinter.Text(self)

            # les boutons OK et Annuler :
            okButton = tkinter.Button(self, text=okButtonText, command=self.doOk)
            cancelButton = tkinter.Button(self, text=cancelButtonText, command=root.destroy)

            # on agence tout ça :
            self.editor.pack(side="top")
            cancelButton.pack(side="right")
            okButton.pack(side="right")

            # le titre de la fenêtre :
            self.master.title(windowTitleText)

            # redimensionnement de la fenêtre :
            self.master.minsize(windowWidth, windowHeight)

            # on vide le texte précédent :
            saveText()

        def doOk(self):
            contents = self.editor.get(1.0, tkinter.END)
            saveText(contents)
            root.destroy()



# **************************************************
#       FONCTION PRINCIPALE (MAIN)
# **************************************************
if __name__ == '__main__':
    if GUI in ('PYSIDE', 'PYQT'):
        app = QtGui.QApplication(sys.argv)
        myWindow = MyWindow()
        myWindow.show()
        sys.exit(app.exec_())
    elif GUI == 'TKINTER':
        root = tkinter.Tk()
        app = Application(master=root)
        app.mainloop()
    else:
        print('PAS DE GUI DISPONIBLE !!!')

